#!/bin/sh
# ==================================================================
# POV-Ray 3.6 - Unix source version - KDE install script
# ==================================================================
# written July 2003 - March 2004 by Christoph Hormann
# Based on parts of the Linux binary version install script
# This file is part of POV-Ray and subject to the POV-Ray licence
# see POVLEGAL.DOC for details.
# ==================================================================


# @@KDE_BEGIN@@

PACKAGE=megapov
VERSION=1.1
VER_DIR=${PACKAGE}-$VERSION
DEFAULT_DIR=/usr/local
SYSCONFDIR=$DEFAULT_DIR/etc

# ==================================================================
#    Add read+write path to user povray.conf file
# ==================================================================

add_readwrite_conf()
{
  DIR_NAME=$1
  CONF_FILE="$HOME/.${PACKAGE}/$VERSION/povray.conf"

  echo "  checking conf file $CONF_FILE"

  if [ ! -f "$CONF_FILE" ] ; then
		cp -f "$SYSCONFDIR/povray.conf" "$CONF_FILE"
  fi

  if [ -w "$CONF_FILE" ] ; then

		if grep -E -i "$DIR_NAME" "$CONF_FILE" > /dev/null ; then
			echo "    - file does not need to be modified"
		else
			echo "    - adding new read+write path"

			cp -f "$CONF_FILE" "$CONF_FILE.bak"

			grep -B 1000 -E -i "^\[Permitted Paths\]" "$CONF_FILE.bak" > "$CONF_FILE" 2> /dev/null

			echo ";--- Lines added by MegaPOV $VERSION install script ---" >> "$CONF_FILE"
			echo "read+write* =  \"$DIR_NAME\"" >> "$CONF_FILE"
			echo ";---------------------------------------------------" >> "$CONF_FILE"

			grep -A 1000 -E -i "^\[Permitted Paths\]" "$CONF_FILE.bak" | sed "/^\[Permitted Paths\]/d" >> "$CONF_FILE" 2> /dev/null

			rm -f "$CONF_FILE.bak"

		fi

  else
    echo "Error: could not modify povray.conf"
  fi
}

# ==================================================================
#    Add read path to user povray.conf file
# ==================================================================

add_read_conf()
{
  DIR_NAME=$1
  CONF_FILE="$HOME/.${PACKAGE}/$VERSION/povray.conf"

  echo "  checking conf file $CONF_FILE"

  if [ ! -f "$CONF_FILE" ] ; then
		cp -f "$SYSCONFDIR/povray.conf" "$CONF_FILE"
  fi

  if [ -w "$CONF_FILE" ] ; then

		if grep -E -i "$DIR_NAME" "$CONF_FILE" > /dev/null ; then
			echo "    - file does not need to be modified"
		else
			echo "    - adding new read path"

			cp -f "$CONF_FILE" "$CONF_FILE.bak"

			grep -B 1000 -E -i "^\[Permitted Paths\]" "$CONF_FILE.bak" > "$CONF_FILE" 2> /dev/null

			echo ";--- Lines added by MegaPOV $VERSION install script ---" >> "$CONF_FILE"
			echo "read* =  \"$DIR_NAME\"" >> "$CONF_FILE"
			echo ";---------------------------------------------------" >> "$CONF_FILE"

			grep -A 1000 -E -i "^\[Permitted Paths\]" "$CONF_FILE.bak" | sed "/^\[Permitted Paths\]/d" >> "$CONF_FILE" 2> /dev/null

			rm -f "$CONF_FILE.bak"

		fi

  else
    echo "Error: could not modify povray.conf"
  fi
}

# ==================================================================
#    Determine installation dir from Library_path settings in ini
# ==================================================================

install_dir()
{
  if [ -z "$POVINI" ] ; then
    test -f "$SYSCONFDIR/povray.ini" && POVINI="$SYSCONFDIR/povray.ini"
    test -f "$HOME/.povrayrc" && POVINI="$HOME/.povrayrc"
    test -f "$SYSCONFDIR/${PACKAGE}/$VERSION/povray.ini" && POVINI="$SYSCONFDIR/${PACKAGE}/$VERSION/povray.ini"
    test -f "$HOME/.${PACKAGE}/$VERSION/povray.ini" && POVINI="$HOME/.${PACKAGE}/$VERSION/povray.ini"
  fi

  if [ ! -z "$POVINI" ] ; then
    # this is not a completely failsafe method but it should work in most cases
    INSTALL_DIR=`grep -E -i "^library_path=.*share/povray-3.6" "$POVINI" | head -n 1 | sed "s?[^=]*=\"*??;s?/share/povray-3.6.*??"`
		if [ -z "$INSTALL_DIR" ] ; then
			INSTALL_DIR=`grep -E -i "^library_path=.*$VER_DIR" "$POVINI" | head -n 1 | sed "s?[^=]*=\"*??;s?/share/$VER_DIR.*??"`
		fi
    echo "$INSTALL_DIR"
  fi
}

# ==================================================================
#    Add file name to install log
# ==================================================================

log_install()
{
  if [ -w "$DEFAULT_DIR/share/$VER_DIR/" ] ; then
		LOG_NAME="$DEFAULT_DIR/share/$VER_DIR/install.log"
	else
    if [ -w "$HOME/.${PACKAGE}/$VERSION/" ] ; then
			LOG_NAME="$HOME/.${PACKAGE}/$VERSION/install.log"
		else
		  return 0
		fi
  fi

  if [ -z "$1$2" ] ; then
		rm -f "$LOG_NAME"
	fi

	if [ ! -f "$LOG_NAME" ] ; then
	  echo "# MegaPOV version $VERSION install log" > "$LOG_NAME"
	  echo "# started `date`" >> "$LOG_NAME"
	fi

  if [ "$1" = "B" ] ; then
		FILE_NAME=`echo "$2" | sed "s? ?%?g"`
    FILE_NAME="$FILE_NAME $3"
  else 
		FILE_NAME=`echo "$2" | sed "s? ?%?g"`
	fi

	echo "$1 $FILE_NAME" >> "$LOG_NAME"
}

# ==================================================================
#    install KDE panel entries
# ==================================================================

kde_install()
{
  if [ -z "$1" ] ; then
    INSTALL_DIR=`install_dir`
  else
    INSTALL_DIR="$1"
  fi

  if [ -z "$INSTALL_DIR" ] ; then
    echo "------------------------------------------------------"
    echo "KDE integration NOT successful."
    echo "The directory where POV-Ray is installed could not be" 
    echo "determined.  Make sure POV-Ray is correctly installed"
    echo "on this computer"
    echo "------------------------------------------------------"
    return 0
  fi

  echo "------------------------------------------------------"
  echo "installing KDE integration for user '$USER'..."

  if [ -z "$KDEHOME" ] ; then 
    if [ -d "$HOME/.kde" ] ; then 
      KDEHOME="$HOME/.kde"
    else
      echo "could not determine user KDEHOME directory."
      echo "make sure KDE is correctly installed"
      return 0
    fi
  else
    if [ ! -d "$KDEHOME" ] ; then
      echo "user KDEHOME directory ($KDEHOME) does not exist"
      return 0
    fi
  fi

  if [ ! -w "$KDEHOME" ] ; then
    echo "no write permission for user KDEHOME directory ($KDEHOME)"
    return 0
  fi

  test -d "$KDEHOME/share" || mkdir "$KDEHOME/share"

  if [ -d "$INSTALL_DIR/share/$VER_DIR/icons" ] ; then

    echo "  copying POV-Ray icons..."

    test -d "$KDEHOME/share/icons" || mkdir "$KDEHOME/share/icons"

		ICON_SIZES="16 32 48 64"

		for ICON_SIZE in $ICON_SIZES ; do

			test -d "$KDEHOME/share/icons/hicolor/${ICON_SIZE}x${ICON_SIZE}/apps" || mkdir "$KDEHOME/share/icons/hicolor/${ICON_SIZE}x${ICON_SIZE}/apps"
			cp -f "$INSTALL_DIR/share/$VER_DIR/icons/${PACKAGE}_${ICON_SIZE}.png" "$KDEHOME/share/icons/hicolor/${ICON_SIZE}x${ICON_SIZE}/apps/${PACKAGE}.png"
			log_install "F" "$KDEHOME/share/icons/hicolor/${ICON_SIZE}x${ICON_SIZE}/apps/${PACKAGE}.png"

		done

  else

    echo "Could not find required files, make sure MegaPOV $VERSION is correctly installed"
    echo ""

  fi

  echo "  installing main MegaPOV $VERSION submenu..."

  if [ ! -d "$KDEHOME/share/applnk" ] ; then 
		mkdir -p "$KDEHOME/share/applnk"
    KDE_PANEL_DIR="$KDEHOME/share/applnk/$VER_DIR"
  else
    KDE_PANEL_DIR="$KDEHOME/share/applnk/$VER_DIR"
  fi

  if [ -d "$KDE_PANEL_DIR" ] ; then 
    rm -rf $KDE_PANEL_DIR/*
  else
    mkdir "$KDE_PANEL_DIR"
  fi

	log_install "F" "$KDE_PANEL_DIR"

  echo "[Desktop Entry]
Name=MegaPOV $VERSION
Icon=${PACKAGE}
" > "$KDE_PANEL_DIR/.directory"

  echo "  installing ini file link..."

  if [ -f "$HOME/.${PACKAGE}/$VERSION/povray.ini" ] ; then
    POVINI="$HOME/.${PACKAGE}/$VERSION/povray.ini"
    echo "[Desktop Entry]
Type=Application
Exec=kwrite $POVINI
Icon=txt
Name=edit user povray.ini file (~/.${PACKAGE}/$VERSION/povray.ini)
" > "$KDE_PANEL_DIR/ini.desktop"
  else
    POVINI="$SYSCONFDIR/${PACKAGE}/$VERSION/povray.ini"
    echo "[Desktop Entry]
Type=Application
Exec=kdesu kwrite $POVINI
Icon=txt
Name=edit global povray.ini file
" > "$KDE_PANEL_DIR/ini.desktop"
  fi

  echo "  installing configuration file link..."

  if [ -f "$HOME/.${PACKAGE}/$VERSION/povray.conf" ] ; then
    POVCONF="$HOME/.${PACKAGE}/$VERSION/povray.conf"
    echo "[Desktop Entry]
Type=Application
Exec=kwrite $POVCONF
Icon=txt
Name=edit IO-restrictions configuration file (~/.${PACKAGE}/$VERSION/povray.conf)
" > "$KDE_PANEL_DIR/conf_user.desktop"
  fi

  if [ -f "$SYSCONFDIR/${PACKAGE}/$VERSION/povray.conf" ] ; then
    POVCONF="$SYSCONFDIR/${PACKAGE}/$VERSION/povray.conf"
    echo "[Desktop Entry]
Type=Application
Exec=kdesu kwrite $POVCONF
Icon=txt
Name=edit global IO-restrictions configuration file
" > "$KDE_PANEL_DIR/conf_sys.desktop"
  fi

  echo "  installing documentation link..."

  echo "[Desktop Entry]
Type=Application
Exec=konqueror $INSTALL_DIR/share/doc/$VER_DIR/html/index.html
Icon=html
Name=Documentation
" > "$KDE_PANEL_DIR/docu.desktop"

  echo "[Desktop Entry]
Type=Application
Exec=konqueror $INSTALL_DIR/share/doc/$VER_DIR/html/povlegal.html
Icon=html
Name=The POV-Ray licence (POVLEGAL.DOC)
" > "$KDE_PANEL_DIR/povlegal.desktop"


  # needs an extra invitation
	if [ -d "$KDEHOME/share/applnk-redhat" ] ; then 
    KDE_RH_PANEL_DIR="$KDEHOME/share/applnk-redhat/$VER_DIR"
		if [ -L "$KDE_RH_PANEL_DIR" ] ; then 
		  rm "$KDE_RH_PANEL_DIR"
		fi
		if [ -d "$KDE_RH_PANEL_DIR" ] ; then 
		  rm -rf "$KDE_RH_PANEL_DIR"
		fi
		ln -s "$KDE_PANEL_DIR" "$KDE_RH_PANEL_DIR"
		log_install "F" "$KDE_RH_PANEL_DIR"
  fi

  echo "Finished installing KDE panel entries"
  echo "------------------------------------------------------"
  echo ""

  return 1
}

# @@KDE_END@@


kde_install

