/****************************************************************************
 *                  hdr.h
 *
 *  Author: Mael, Christoph Hormann
 *
 *  High Dynamic Range Images reading/writing function
 *
 *
 *  Writing code derived from PPM code
 *
 *  based on RADIANCE HDR image specs:
 *
 *    http://radsite.lbl.gov/radiance/refer/Notes/picture_format.html
 *
 * from Persistence of Vision(tm) Ray Tracer version 3.6.
 * Copyright 1991-2003 Persistence of Vision Team
 * Copyright 2003-2004 Persistence of Vision Raytracer Pty. Ltd.
 *---------------------------------------------------------------------------
 * NOTICE: This source code file is provided so that users may experiment
 * with enhancements to POV-Ray and to port the software to platforms other
 * than those supported by the POV-Ray developers. There are strict rules
 * regarding how you are permitted to use this file. These rules are contained
 * in the distribution and derivative versions licenses which should have been
 * provided with this file.
 *
 * These licences may be found online, linked from the end-user license
 * agreement that is located at http://www.povray.org/povlegal.html
 *---------------------------------------------------------------------------
 * This program is based on the popular DKB raytracer version 2.12.
 * DKBTrace was originally written by David K. Buck.
 * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
 *---------------------------------------------------------------------------
 *
 *===========================================================================
 * This file is part of MegaPOV, a modified and unofficial version of POV-Ray
 * For more information on MegaPOV visit our website:
 * http://megapov.inetart.net/
 *===========================================================================
 *
 * $RCSfile: hdr.h,v $
 * $Revision: 1.10 $
 * $Author: chris $
 *
 *****************************************************************************/ 
 

#ifndef HDR_H
#define HDR_H

#ifdef HDR_PATCH

BEGIN_POV_NAMESPACE
/*****************************************************************************
* Global typedefs
******************************************************************************/

typedef unsigned char RGBE[4]; /* red, green, blue, exponent */

class HDR_Image : public Image_File_Class
{
  public:
    HDR_Image(char *filename, int width, int height, int mode, int line = 0);
    ~HDR_Image();

    void Write_Line(COLOUR *line_data);
    int Read_Line(COLOUR *line_data);

    int Line() { return line_number; };
    int Width() { return width; };
    int Height() { return height; };
  private:
    void ColorToRGBE(RGBE clr, COLC r, COLC g, COLC b);

    char *filename;
    int mode;
    int width, height;
    int line_number;
    IStream *in_file;
    OStream *out_file;
};

/*****************************************************************************
* Global functions
******************************************************************************/

void Read_HDR_Image (IMAGE *Image, char *filename);
END_POV_NAMESPACE

#endif

#endif

/*-ML: hdr_patch*/
