/****************************************************************************
 *                  render.h
 *
 * This module contains all defines, typedefs, and prototypes for RENDER.CPP.
 *
 * from Persistence of Vision(tm) Ray Tracer version 3.6.
 * Copyright 1991-2003 Persistence of Vision Team
 * Copyright 2003-2004 Persistence of Vision Raytracer Pty. Ltd.
 *---------------------------------------------------------------------------
 * NOTICE: This source code file is provided so that users may experiment
 * with enhancements to POV-Ray and to port the software to platforms other
 * than those supported by the POV-Ray developers. There are strict rules
 * regarding how you are permitted to use this file. These rules are contained
 * in the distribution and derivative versions licenses which should have been
 * provided with this file.
 *
 * These licences may be found online, linked from the end-user license
 * agreement that is located at http://www.povray.org/povlegal.html
 *---------------------------------------------------------------------------
 * This program is based on the popular DKB raytracer version 2.12.
 * DKBTrace was originally written by David K. Buck.
 * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
 *---------------------------------------------------------------------------
 *
 *===========================================================================
 * This file is part of MegaPOV, a modified and unofficial version of POV-Ray
 * For more information on MegaPOV visit our website:
 * http://megapov.inetart.net/
 *===========================================================================
 *
 * $RCSfile: render.h,v $
 * $Revision: 1.13 $
 * $Author: chris $
 *
 *****************************************************************************/


#ifndef RENDER_H
#define RENDER_H

BEGIN_POV_NAMESPACE

/*****************************************************************************
* Global preprocessor defines
******************************************************************************/




/*****************************************************************************
* Global typedefs
******************************************************************************/

typedef enum
{
  CSV,
  SYS,
  PPM,
  TARGA,
  PNG,
  NONE
} Histogram_Types;

/*****************************************************************************
* Global variables
******************************************************************************/

extern COLOUR *Previous_Line, *Current_Line, *Temp_Line;

extern char *Previous_Line_Antialiased_Flags, *Current_Line_Antialiased_Flags;

extern unsigned char *Red_Row_255, *Green_Row_255, *Blue_Row_255, *Alpha_Row_255;

extern long SuperSampleCount, RadiosityCount, MosaicPreviewSize;

extern DBL maxclr;

extern int Current_Line_Number;

extern int Trace_Level, Max_Trace_Level, Highest_Trace_Level;
extern bool Had_Max_Trace_Level;
extern DBL Total_Depth;

/* Object-Ray Options [ENB 9/97] */
extern bool In_Reflection_Ray;
extern bool In_Shadow_Ray;

#ifdef NO_RADIOSITY_PATCH /* Christoph Hormann January 2004 */
extern bool In_Radiosity_Ray;
#endif

extern DBL ADC_Bailout;

extern unsigned long *histogram_grid ;
extern unsigned long max_histogram_value ;
extern Image_File_Class *Histogram_File ;

/*****************************************************************************
* Global functions
******************************************************************************/

void Read_Rendered_Part (char *New_Fname);
void Start_Tracing_Radiosity_Preview (int StartPixelSize, int EndPixelSize);
void Start_Tracing_Mosaic_Preview (int StartPixelSize, int EndPixelSize);
void Start_Adaptive_Tracing (
                             #ifdef POST_PROCESS_PATCH
                               DBL *Post_Process_Info
                             #else
                               void
                             #endif
                             );
void Start_Non_Adaptive_Tracing (
                                 #ifdef POST_PROCESS_PATCH
                                   DBL *Post_Process_Info
                                 #else
                                   void
                                 #endif
                                 );
void Initialize_Renderer (void);
void Terminate_Renderer (void);
DBL  Trace (RAY *Ray, COLOUR Colour, DBL Weight
                                               #ifdef POST_PROCESS_PATCH
                                               , DBL *Post_Process_Info = NULL
                                               #endif
                                               );
void Check_User_Abort (int Do_Stats);
void write_histogram (char *filename);
void destroy_histogram (void);
void initialize_ray_container_state(RAY *Ray, int Compute);
#ifdef PIGMENT_CAMERA_VIEW_PATCH
int  create_ray (CAMERA *camera, RAY *ray, DBL x, DBL y, int ray_number);
#endif

#ifdef MOTION_BLUR_PATCH
	DBL  Trace_With_Blur (RAY *Ray, COLOUR Colour, DBL Weight
                                               #ifdef POST_PROCESS_PATCH
                                               , DBL *Post_Process_Info = NULL
                                               #endif
                                               );
#endif
END_POV_NAMESPACE

#endif
