/****************************************************************************
 *                  splines.h
 *
 * Contributed by ???
 *
 * This module contains all defines, typedefs, and prototypes for splines.cpp.
 *
 * from Persistence of Vision(tm) Ray Tracer version 3.6.
 * Copyright 1991-2003 Persistence of Vision Team
 * Copyright 2003-2004 Persistence of Vision Raytracer Pty. Ltd.
 *---------------------------------------------------------------------------
 * NOTICE: This source code file is provided so that users may experiment
 * with enhancements to POV-Ray and to port the software to platforms other
 * than those supported by the POV-Ray developers. There are strict rules
 * regarding how you are permitted to use this file. These rules are contained
 * in the distribution and derivative versions licenses which should have been
 * provided with this file.
 *
 * These licences may be found online, linked from the end-user license
 * agreement that is located at http://www.povray.org/povlegal.html
 *---------------------------------------------------------------------------
 * This program is based on the popular DKB raytracer version 2.12.
 * DKBTrace was originally written by David K. Buck.
 * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
 *---------------------------------------------------------------------------
 *
 *===========================================================================
 * This file is part of MegaPOV, a modified and unofficial version of POV-Ray
 * For more information on MegaPOV visit our website:
 * http://megapov.inetart.net/
 *===========================================================================
 *
 * $RCSfile: splines.h,v $
 * $Revision: 1.12 $
 * $Author: chris $
 *
 *****************************************************************************/


#ifndef SPLINE_H
#define SPLINE_H
/* Generic header for spline modules */

#include "frame.h"

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "config.h"

BEGIN_POV_NAMESPACE

#define INIT_SPLINE_SIZE     16

#define LINEAR_SPLINE         1
#define QUADRATIC_SPLINE      2
#define NATURAL_SPLINE        3
#define CATMULL_ROM_SPLINE    4
#ifdef SOR_SPLINE_PATCH
  #define SOR_LIKE_SPLINE     5
#endif
#ifdef TCB_SPLINE_PATCH
  #define TCB_SPLINE          6
#endif
#ifdef AKIMA_SPLINE_PATCH
  #define AKIMA_SPLINE        7
#endif
#ifdef BASIC_X_SPLINE_PATCH
  #define BASIC_X_SPLINE      8
#endif
#ifdef EXTENDED_X_SPLINE_PATCH
  #define EXTENDED_X_SPLINE   9
#endif
#ifdef GENERAL_X_SPLINE_PATCH
  #define GENERAL_X_SPLINE   10
#endif

SPLINE * Create_Spline(int Type);
SPLINE * Copy_Spline(SPLINE * Old);
void Destroy_Spline(SPLINE * Spline);
void Insert_Spline_Entry(SPLINE * Spline, DBL p, EXPRESS v
                                                          #ifdef TCB_SPLINE_PATCH
                                                          , DBL in_tension
                                                          , DBL out_tension
                                                          , DBL in_continuity
                                                          , DBL out_continuity
                                                          , DBL in_bias
                                                          , DBL out_bias
                                                          #endif
                                                          #if defined( EXTENDED_X_SPLINE_PATCH ) || defined( GENERAL_X_SPLINE_PATCH )
                                                          , DBL freedom_degree
                                                          #endif
                                                          );
DBL Get_Spline_Val(SPLINE * sp, DBL p, EXPRESS v, int *Terms);

END_POV_NAMESPACE

#endif
